@@ -1,31 +1,29 @@
 """HDXObject abstract class containing helper functions for creating, checking, and updating HDX objects.
 New HDX objects should extend this in similar fashion to Resource for example.
 """
-import abc
 import copy
 import logging
+from abc import ABC, abstractmethod
 from collections import UserDict
-from typing import Any, Dict, List, Optional, Tuple, TypeVar, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 from ckanapi.errors import NotFound
-
-from hdx.hdx_configuration import Configuration
 from hdx.utilities.dictandlist import merge_two_dictionaries
 from hdx.utilities.loader import (
     load_json_into_existing_dict,
     load_yaml_into_existing_dict,
 )
 
-logger = logging.getLogger(__name__)
+from hdx.api.configuration import Configuration
 
-HDXObjectUpperBound = TypeVar("HDXObjectUpperBound", bound="HDXObject")
+logger = logging.getLogger(__name__)
 
 
 class HDXError(Exception):
     pass
 
 
-class HDXObject(UserDict):
+class HDXObject(UserDict, ABC):
     """HDXObject abstract class containing helper functions for creating, checking, and updating HDX objects.
     New HDX objects should extend this in similar fashion to Resource for example.
 
@@ -34,27 +32,24 @@ class HDXObject(UserDict):
         configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.
     """
 
-    __metaclass__ = abc.ABCMeta
-
     @staticmethod
-    @abc.abstractmethod
+    @abstractmethod
     def actions() -> Dict[str, str]:
         """Dictionary of actions that can be performed on object
 
         Returns:
             Dict[str, str]: Dictionary of actions that can be performed on object
         """
-        raise NotImplementedError
 
     def __init__(
         self, initial_data: Dict, configuration: Optional[Configuration] = None
     ) -> None:
-        super().__init__(initial_data)
         self.old_data = None
         if configuration is None:
-            self.configuration = Configuration.read()
+            self.configuration: Configuration = Configuration.read()
         else:
-            self.configuration = configuration
+            self.configuration: Configuration = configuration
+        super().__init__(initial_data)
 
     def get_old_data_dict(self) -> None:
         """Get previous internal dictionary
@@ -141,10 +136,10 @@ class HDXObject(UserDict):
         return False
 
     @staticmethod
-    @abc.abstractmethod
+    @abstractmethod
     def read_from_hdx(
         id_field: str, configuration: Optional[Configuration] = None
-    ) -> Optional[HDXObjectUpperBound]:
+    ) -> Optional["HDXObject"]:
         """Abstract method to read the HDX object given by identifier from HDX and return it
 
         Args:
@@ -152,9 +147,8 @@ class HDXObject(UserDict):
             configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.
 
         Returns:
-            Optional[T <= HDXObject]: HDX object if successful read, None if not
+            Optional[HDXObject]: HDX object if successful read, None if not
         """
-        raise NotImplementedError
 
     @classmethod
     def _read_from_hdx_class(
@@ -162,7 +156,7 @@ class HDXObject(UserDict):
         object_type: str,
         identifier: str,
         configuration: Optional[Configuration] = None,
-    ) -> Optional[HDXObjectUpperBound]:
+    ) -> Optional["HDXObject"]:
         """Reads the HDX object given by identifier from HDX and returns it
 
         Args:
@@ -171,7 +165,7 @@ class HDXObject(UserDict):
             configuration (Optional[Configuration]): HDX configuration. Defaults to global configuration.
 
         Returns:
-            Optional[HDXObjectUpperBound]: HDX object if successful read, None if not
+            Optional[HDXObject]: HDX object if successful read, None if not
         """
         hdxobject = cls(configuration=configuration)
         result = hdxobject._load_from_hdx(object_type, identifier)
@@ -179,11 +173,15 @@ class HDXObject(UserDict):
             return hdxobject
         return None
 
-    def _check_existing_object(self, object_type: str, id_field_name: str) -> None:
+    def _check_existing_object(
+        self, object_type: str, id_field_name: str
+    ) -> None:
         if not self.data:
             raise HDXError(f"No data in {object_type}!")
         if id_field_name not in self.data:
-            raise HDXError(f"No {id_field_name} field (mandatory) in {object_type}!")
+            raise HDXError(
+                f"No {id_field_name} field (mandatory) in {object_type}!"
+            )
 
     def _check_load_existing_object(
         self, object_type: str, id_field_name: str, operation: str = "update"
@@ -202,7 +200,7 @@ class HDXObject(UserDict):
         if not self._load_from_hdx(object_type, self.data[id_field_name]):
             raise HDXError(f"No existing {object_type} to {operation}!")
 
-    @abc.abstractmethod
+    @abstractmethod
     def check_required_fields(self, ignore_fields: List[str] = list()) -> None:
         """Abstract method to check that metadata for HDX object is complete. The parameter ignore_fields should
         be set if required to any fields that should be ignored for the particular operation.
@@ -213,7 +211,6 @@ class HDXObject(UserDict):
         Returns:
             None
         """
-        raise NotImplementedError
 
     def _check_required_fields(
         self, object_type: str, ignore_fields: List[str]
@@ -229,8 +226,12 @@ class HDXObject(UserDict):
         for field in self.configuration[object_type]["required_fields"]:
             if field not in ignore_fields:
                 if field not in self.data:
-                    raise HDXError(f"Field {field} is missing in {object_type}!")
-                if not self.data[field] and not isinstance(self.data[field], bool):
+                    raise HDXError(
+                        f"Field {field} is missing in {object_type}!"
+                    )
+                if not self.data[field] and not isinstance(
+                    self.data[field], bool
+                ):
                     raise HDXError(f"Field {field} is empty in {object_type}!")
 
     def _check_kwargs_fields(self, object_type: str, **kwargs: Any) -> None:
@@ -253,16 +254,18 @@ class HDXObject(UserDict):
             "skip_validation" in kwargs
         ):  # Whether or not CKAN should perform validation steps (checking fields present)
             self.data["skip_validation"] = kwargs["skip_validation"]
-        ignore_fields = kwargs.get("ignore_fields", list())
-        ignore_field = self.configuration[object_type].get("ignore_on_update")
-        if ignore_field and ignore_field not in ignore_fields:
-            ignore_fields.append(ignore_field)
-        ignore_field = kwargs.get("ignore_field")
-        if ignore_field and ignore_field not in ignore_fields:
-            ignore_fields.append(ignore_field)
         if "ignore_check" not in kwargs or not kwargs.get(
             "ignore_check"
         ):  # allow ignoring of field checks
+            ignore_fields = kwargs.get("ignore_fields", list())
+            ignore_field = self.configuration[object_type].get(
+                "ignore_on_update"
+            )
+            if ignore_field and ignore_field not in ignore_fields:
+                ignore_fields.append(ignore_field)
+            ignore_field = kwargs.get("ignore_field")
+            if ignore_field and ignore_field not in ignore_fields:
+                ignore_fields.append(ignore_field)
             self.check_required_fields(ignore_fields=ignore_fields)
 
     def _hdx_update(
@@ -289,7 +292,9 @@ class HDXObject(UserDict):
         """
         self._check_kwargs_fields(object_type, **kwargs)
         operation = kwargs.get("operation", "update")
-        self._save_to_hdx(operation, id_field_name, files_to_upload, force_active)
+        self._save_to_hdx(
+            operation, id_field_name, files_to_upload, force_active
+        )
 
     def _merge_hdx_update(
         self,
@@ -322,14 +327,13 @@ class HDXObject(UserDict):
             **kwargs,
         )
 
-    @abc.abstractmethod
+    @abstractmethod
     def update_in_hdx(self) -> None:
         """Abstract method to check if HDX object exists in HDX and if so, update it
 
         Returns:
             None
         """
-        raise NotImplementedError
 
     def _update_in_hdx(
         self,
@@ -395,7 +399,9 @@ class HDXObject(UserDict):
                 idstr = f" {data[id_field_name]}"
             else:
                 idstr = ""
-            raise HDXError(f"Failed when trying to {action}{idstr}! (POST)") from e
+            raise HDXError(
+                f"Failed when trying to {action}{idstr}! (POST)"
+            ) from e
         finally:
             for file in files_to_upload.values():
                 if isinstance(file, str):
@@ -423,18 +429,19 @@ class HDXObject(UserDict):
         """
         if force_active:
             self.data["state"] = "active"
-        result = self._write_to_hdx(action, self.data, id_field_name, files_to_upload)
+        result = self._write_to_hdx(
+            action, self.data, id_field_name, files_to_upload
+        )
         self.old_data = self.data
         self.data = result
 
-    @abc.abstractmethod
+    @abstractmethod
     def create_in_hdx(self) -> None:
         """Abstract method to check if resource exists in HDX and if so, update it, otherwise create it
 
         Returns:
             None
         """
-        raise NotImplementedError
 
     def _create_in_hdx(
         self,
@@ -463,21 +470,28 @@ class HDXObject(UserDict):
         if id_field_name in self.data and self._load_from_hdx(
             object_type, self.data[id_field_name]
         ):
-            logger.warning(f"{object_type} exists. Updating {self.data[id_field_name]}")
+            logger.warning(
+                f"{object_type} exists. Updating {self.data[id_field_name]}"
+            )
             self._merge_hdx_update(
-                object_type, id_field_name, files_to_upload, force_active, **kwargs
+                object_type,
+                id_field_name,
+                files_to_upload,
+                force_active,
+                **kwargs,
             )
         else:
-            self._save_to_hdx("create", name_field_name, files_to_upload, force_active)
+            self._save_to_hdx(
+                "create", name_field_name, files_to_upload, force_active
+            )
 
-    @abc.abstractmethod
+    @abstractmethod
     def delete_from_hdx(self) -> None:
         """Abstract method to deletes a resource from HDX
 
         Returns:
             None
         """
-        raise NotImplementedError
 
     def _delete_from_hdx(self, object_type: str, id_field_name: str) -> None:
         """Helper method to deletes a resource from HDX
@@ -490,7 +504,9 @@ class HDXObject(UserDict):
             None
         """
         if id_field_name not in self.data:
-            raise HDXError(f"No {id_field_name} field (mandatory) in {object_type}!")
+            raise HDXError(
+                f"No {id_field_name} field (mandatory) in {object_type}!"
+            )
         self._save_to_hdx("delete", id_field_name)
 
     @classmethod
@@ -520,20 +536,20 @@ class HDXObject(UserDict):
 
     def _addupdate_hdxobject(
         self,
-        hdxobjects: List[HDXObjectUpperBound],
+        hdxobjects: List["HDXObject"],
         id_field: str,
-        new_hdxobject: HDXObjectUpperBound,
-    ) -> HDXObjectUpperBound:
+        new_hdxobject: "HDXObject",
+    ) -> "HDXObject":
         """Helper function to add a new HDX object to a supplied list of HDX objects or update existing metadata if the object
         already exists in the list
 
         Args:
-            hdxobjects (List[T <= HDXObject]): list of HDX objects to which to add new objects or update existing ones
+            hdxobjects (List[HDXObject]): list of HDX objects to which to add new objects or update existing ones
             id_field (str): Field on which to match to determine if object already exists in list
-            new_hdxobject (T <= HDXObject): The HDX object to be added/updated
+            new_hdxobject (HDXObject): The HDX object to be added/updated
 
         Returns:
-            T <= HDXObject: The HDX object which was added or updated
+            HDXObject: The HDX object which was added or updated
         """
         for hdxobject in hdxobjects:
             if hdxobject[id_field] == new_hdxobject[id_field]:
@@ -544,16 +560,16 @@ class HDXObject(UserDict):
 
     def _remove_hdxobject(
         self,
-        objlist: List[Union[HDXObjectUpperBound, Dict]],
-        obj: Union[HDXObjectUpperBound, Dict, str],
+        objlist: List[Union["HDXObject", Dict]],
+        obj: Union["HDXObject", Dict, str],
         matchon: str = "id",
         delete: bool = False,
     ) -> bool:
         """Remove an HDX object from a list within the parent HDX object
 
         Args:
-            objlist (List[Union[T <= HDXObject,Dict]]): list of HDX objects
-            obj (Union[T <= HDXObject,Dict,str]): Either an id or hdx object metadata either from an HDX object or a dictionary
+            objlist (List[Union[HDXObject,Dict]]): list of HDX objects
+            obj (Union[HDXObject,Dict,str]): Either an id or hdx object metadata either from an HDX object or a dictionary
             matchon (str): Field to match on. Defaults to id.
             delete (bool): Whether to delete HDX object. Defaults to False.
 
@@ -579,13 +595,11 @@ class HDXObject(UserDict):
                 return True
         return False
 
-    def _convert_hdxobjects(
-        self, hdxobjects: List[HDXObjectUpperBound]
-    ) -> List[HDXObjectUpperBound]:
+    def _convert_hdxobjects(self, hdxobjects: List["HDXObject"]) -> List[Dict]:
         """Helper function to convert supplied list of HDX objects to a list of dict
 
         Args:
-            hdxobjects (List[T <= HDXObject]): List of HDX objects to convert
+            hdxobjects (List[HDXObject]): List of HDX objects to convert
 
         Returns:
             List[Dict]: List of HDX objects converted to simple dictionaries
@@ -597,19 +611,19 @@ class HDXObject(UserDict):
 
     def _copy_hdxobjects(
         self,
-        hdxobjects: List[HDXObjectUpperBound],
+        hdxobjects: List["HDXObject"],
         hdxobjectclass: type,
         attribute_to_copy: Optional[str] = None,
-    ) -> List[HDXObjectUpperBound]:
+    ) -> List["HDXObject"]:
         """Helper function to make a deep copy of a supplied list of HDX objects
 
         Args:
-            hdxobjects (List[T <= HDXObject]): list of HDX objects to copy
+            hdxobjects (List[HDXObject]): list of HDX objects to copy
             hdxobjectclass (type): Type of the HDX Objects to be copied
             attribute_to_copy (Optional[str]): An attribute to copy over from the HDX object. Defaults to None.
 
         Returns:
-            List[T <= HDXObject]: Deep copy of list of HDX objects
+            List[HDXObject]: Deep copy of list of HDX objects
         """
         newhdxobjects = list()
         for hdxobject in hdxobjects:
@@ -625,7 +639,7 @@ class HDXObject(UserDict):
 
     def _separate_hdxobjects(
         self,
-        hdxobjects: List[HDXObjectUpperBound],
+        hdxobjects: List["HDXObject"],
         hdxobjects_name: str,
         id_field: str,
         hdxobjectclass: type,
@@ -635,7 +649,7 @@ class HDXObject(UserDict):
         the internal dictionary is then deleted.
 
         Args:
-            hdxobjects (List[T <= HDXObject]): list of HDX objects to which to add new objects or update existing ones
+            hdxobjects (List[HDXObject]): list of HDX objects to which to add new objects or update existing ones
             hdxobjects_name (str): Name of key in internal dictionary from which to obtain list of HDX objects
             id_field (str): Field on which to match to determine if object already exists in list
             hdxobjectclass (type): Type of the HDX Object to be added/updated
@@ -644,7 +658,7 @@ class HDXObject(UserDict):
             None
         """
         new_hdxobjects = self.data.get(hdxobjects_name, list())
-        """:type : List[HDXObjectUpperBound]"""
+        """:type : List["HDXObject"]"""
         if new_hdxobjects:
             hdxobject_names = set()
             for hdxobject in hdxobjects:
@@ -657,7 +671,9 @@ class HDXObject(UserDict):
             for new_hdxobject in new_hdxobjects:
                 if new_hdxobject[id_field] not in hdxobject_names:
                     hdxobjects.append(
-                        hdxobjectclass(new_hdxobject, configuration=self.configuration)
+                        hdxobjectclass(
+                            new_hdxobject, configuration=self.configuration
+                        )
                     )
             del self.data[hdxobjects_name]
 
@@ -747,7 +763,9 @@ class HDXObject(UserDict):
         self.data[field] = strings
         return True
 
-    def _add_strings_to_commastring(self, field: str, strings: List[str]) -> bool:
+    def _add_strings_to_commastring(
+        self, field: str, strings: List[str]
+    ) -> bool:
         """Add a list of strings to a comma separated list of strings
 
         Args:
